@@ -250,12 +250,17 @@ class _LLMClient(BaseModel):
             return response
 
         if self.cache:
+            # print(f"DEBUG: Checking cache for prompt with {len(context.prompt.messages)} messages")
             cached = self.cache.get(context.prompt)
             if cached:
+                # print(f"DEBUG: Cache HIT! Returning cached response")
                 cached.local_cache_hit = True
                 if context.status_tracker:
                     context.status_tracker.task_succeeded(context.task_id)
                 return _maybe_postprocess(cached)
+            else:
+                # print(f"DEBUG: Cache MISS")
+                pass
 
         # Execute single request
         assert context.status_tracker
@@ -267,6 +272,7 @@ class _LLMClient(BaseModel):
             context.status_tracker.task_succeeded(context.task_id)
             # Cache successful responses immediately
             if self.cache and response.completion:
+                # print(f"DEBUG: Caching successful response")
                 self.cache.put(context.prompt, response)
             # Call callback if provided
             context.maybe_callback(response, context.status_tracker)
@@ -351,6 +357,8 @@ class _LLMClient(BaseModel):
         prompts = prompts_to_conversations(prompts)
         ids = list(range(len(prompts)))
         results: list[APIResponse | None] = [None for _ in range(len(prompts))]
+        contexts: list[RequestContext | None] = [None for _ in range(len(prompts))]
+        inflight_tasks: set[asyncio.Task[None]] = set()
         # Use existing tracker if client has been opened; otherwise open/close automatically
         tracker: StatusTracker
         tracker_preopened = self._tracker is not None
@@ -413,12 +421,14 @@ class _LLMClient(BaseModel):
                 )
 
                 # Launch simplified request processing
+                contexts[next_context.task_id] = next_context
+
                 async def process_and_store(ctx: RequestContext):
                     try:
                         response = await self.process_single_request(ctx, retry_queue)
                         results[ctx.task_id] = response
-                    except Exception as e:
-                        # Create an error response for validation errors and other exceptions
+                    except BaseException as exc:
+                        # Capture cancellations and other BaseExceptions before fallback response fires.
                         error_response = APIResponse(
                             id=ctx.task_id,
                             model_internal=ctx.model_name,
@@ -426,14 +436,17 @@ class _LLMClient(BaseModel):
                             sampling_params=ctx.sampling_params,
                             status_code=None,
                             is_error=True,
-                            error_message=str(e),
+                            error_message=f"{type(exc).__name__}: {exc}",
+                            raw_response={"exception_repr": repr(exc)},
                         )
                         results[ctx.task_id] = error_response
-                        # Mark task as completed so the main loop can finish
                         if ctx.status_tracker:
                             ctx.status_tracker.task_failed(ctx.task_id)
+                        raise
 
-                asyncio.create_task(process_and_store(next_context))
+                task = asyncio.create_task(process_and_store(next_context))
+                inflight_tasks.add(task)
+                task.add_done_callback(inflight_tasks.discard)
                 next_context = None  # Reset after successful dispatch
                 next_is_retry = False
 
@@ -450,9 +463,37 @@ class _LLMClient(BaseModel):
             # Yield briefly to allow in-flight tasks to progress
             await asyncio.sleep(min(0.01, seconds_to_sleep_each_loop))
 
+        if inflight_tasks:
+            await asyncio.gather(*inflight_tasks, return_exceptions=True)
+
         if not tracker_preopened:
             self.close()
 
+        for idx, response in enumerate(results):
+            if response is None:
+                ctx = contexts[idx]
+                prompt = ctx.prompt if ctx else prompts[idx]
+                sampling_params = (
+                    ctx.sampling_params
+                    if ctx
+                    else self.sampling_params[0]
+                    if self.sampling_params
+                    else SamplingParams()
+                )
+                model_name = ctx.model_name if ctx else self.model_names[0]
+                assert isinstance(
+                    prompt, Conversation
+                ), "expected prompt to be a conversation"
+                results[idx] = APIResponse(
+                    id=idx,
+                    model_internal=model_name,
+                    prompt=prompt,
+                    sampling_params=sampling_params,
+                    status_code=None,
+                    is_error=True,
+                    error_message="Internal error: no response produced.",
+                )
+
         if return_completions_only:
             return [r.completion if r is not None else None for r in results]
 
